@@ -77,6 +77,13 @@ class Unparser:
         for stmt in tree.body:
             self.dispatch(stmt)
 
+    def _Interactive(self, tree):
+        for stmt in tree.body:
+            self.dispatch(stmt)
+
+    def _Expression(self, tree):
+        self.dispatch(tree.body)
+
     # stmt
     def _Expr(self, tree):
         self.fill()
@@ -286,16 +293,17 @@ class Unparser:
                 if comma: self.write(", ")
                 else: comma = True
                 self.dispatch(e)
-            if t.starargs:
-                if comma: self.write(", ")
-                else: comma = True
-                self.write("*")
-                self.dispatch(t.starargs)
-            if t.kwargs:
-                if comma: self.write(", ")
-                else: comma = True
-                self.write("**")
-                self.dispatch(t.kwargs)
+            if sys.version_info[:2] < (3, 5):
+                if t.starargs:
+                    if comma: self.write(", ")
+                    else: comma = True
+                    self.write("*")
+                    self.dispatch(t.starargs)
+                if t.kwargs:
+                    if comma: self.write(", ")
+                    else: comma = True
+                    self.write("**")
+                    self.dispatch(t.kwargs)
             self.write(")")
         elif t.bases:
                 self.write("(")
@@ -307,12 +315,12 @@ class Unparser:
         self.dispatch(t.body)
         self.leave()
 
-    def _FunctionDef(self, t):
+    def _generic_FunctionDef(self, t, async=False):
         self.write("\n")
         for deco in t.decorator_list:
             self.fill("@")
             self.dispatch(deco)
-        self.fill("def "+t.name + "(")
+        self.fill(("async " if async else "") + "def " + t.name + "(")
         self.dispatch(t.args)
         self.write(")")
         if getattr(t, "returns", False):
@@ -322,8 +330,14 @@ class Unparser:
         self.dispatch(t.body)
         self.leave()
 
-    def _For(self, t):
-        self.fill("for ")
+    def _FunctionDef(self, t):
+        self._generic_FunctionDef(t)
+
+    def _AsyncFunctionDef(self, t):
+        self._generic_FunctionDef(t, async=True)
+
+    def _generic_For(self, t, async=False):
+        self.fill("async for " if async else "for ")
         self.dispatch(t.target)
         self.write(" in ")
         self.dispatch(t.iter)
@@ -336,6 +350,12 @@ class Unparser:
             self.dispatch(t.orelse)
             self.leave()
 
+    def _For(self, t):
+        self._generic_For(t)
+
+    def _AsyncFor(self, t):
+        self._generic_For(t, async=True)
+
     def _If(self, t):
         self.fill("if ")
         self.dispatch(t.test)
@@ -370,8 +390,8 @@ class Unparser:
             self.dispatch(t.orelse)
             self.leave()
 
-    def _With(self, t):
-        self.fill("with ")
+    def _generic_With(self, t, async=False):
+        self.fill("async with " if async else "with ")
         if hasattr(t, 'items'):
             interleave(lambda: self.write(", "), self.dispatch, t.items)
         else:
@@ -383,6 +403,12 @@ class Unparser:
         self.dispatch(t.body)
         self.leave()
 
+    def _With(self, t):
+        self._generic_With(t)
+
+    def _AsyncWith(self, t):
+        self._generic_With(t, async=True)
+
     # expr
     def _Bytes(self, t):
         self.write(repr(t.s))
@@ -528,7 +554,8 @@ class Unparser:
 
     binop = { "Add":"+", "Sub":"-", "Mult":"*", "Div":"/", "Mod":"%",
                     "LShift":"<<", "RShift":">>", "BitOr":"|", "BitXor":"^", "BitAnd":"&",
-                    "FloorDiv":"//", "Pow": "**"}
+                    "FloorDiv":"//", "Pow": "**",
+                    "MatMult":"@"}
     def _BinOp(self, t):
         self.write("(")
         self.dispatch(t.left)
@@ -575,16 +602,17 @@ class Unparser:
             if comma: self.write(", ")
             else: comma = True
             self.dispatch(e)
-        if t.starargs:
-            if comma: self.write(", ")
-            else: comma = True
-            self.write("*")
-            self.dispatch(t.starargs)
-        if t.kwargs:
-            if comma: self.write(", ")
-            else: comma = True
-            self.write("**")
-            self.dispatch(t.kwargs)
+        if sys.version_info[:2] < (3, 5):
+            if t.starargs:
+                if comma: self.write(", ")
+                else: comma = True
+                self.write("*")
+                self.dispatch(t.starargs)
+            if t.kwargs:
+                if comma: self.write(", ")
+                else: comma = True
+                self.write("**")
+                self.dispatch(t.kwargs)
         self.write(")")
 
     def _Subscript(self, t):
@@ -680,8 +708,12 @@ class Unparser:
                     self.dispatch(t.kwargannotation)
 
     def _keyword(self, t):
-        self.write(t.arg)
-        self.write("=")
+        if t.arg is None:
+            # starting from Python 3.5 this denotes a kwargs part of the invocation
+            self.write("**")
+        else:
+            self.write(t.arg)
+            self.write("=")
         self.dispatch(t.value)
 
     def _Lambda(self, t):
@@ -703,6 +735,14 @@ class Unparser:
             self.write(" as ")
             self.dispatch(t.optional_vars)
 
+    def _Await(self, t):
+        self.write("(")
+        self.write("await")
+        if t.value:
+            self.write(" ")
+            self.dispatch(t.value)
+        self.write(")")
+
 def roundtrip(filename, output=sys.stdout):
     if six.PY3:
         with open(filename, "rb") as pyfile:
